@@ -1,5 +1,5 @@
 import threading
-from urllib.parse import urlparse, urlencode, parse_qsl
+from urllib.parse import urlparse, urlencode, parse_qsl, quote
 
 from seleniumwire import webdriver
 
@@ -226,3 +226,10 @@ class ProcessHelper:
         except Exception as err:
             logger.error("Something went wrong quitting chromedriver!")
             logger.error(str(err.__class__) + str(err))
+
+    @staticmethod
+    def gen_filename_from_url(url):
+        parsed_url = urlparse(url)
+        url_filename = parsed_url.netloc.replace(".", "-")
+        url_filename += parsed_url.path.replace("/", "-")
+        return quote(url_filename)
