@@ -11,7 +11,7 @@ SocialLoginInformation = {
                                     "adwords.google.", "picasa.google.", "books.google.", "edu.google.", "news.google.",
                                     "developers.google.", "mymaps.google.", "workspace.google.", "gsuite.google.",
                                     "mijnaccount.google.", "scholar.google.", "blog.google", "patents.google.",
-                                    "googlegroups."],
+                                    "googlegroups.", "myaccount.google.com"],
         "extra_texts": ["ورود با حساب گوگل"]
     },
     "Facebook": {
