@@ -1,4 +1,8 @@
-import threading
+import math
+import os
+import shutil
+import tempfile
+import time
 from multiprocessing import Process
 from time import sleep
 
@@ -19,21 +23,32 @@ from processes import generic_process
 from processes.generic_process import GenericAnalysisProcess
 from processes.process_helper import ProcessHelper
 from processes.ssolandscapeanalysis import startpage_search, duck_duck_go_search, bing_search
+from processes.ssolandscapeanalysis.logorecognition.logo_recognizer import LogoReconizer
 from processes.ssolandscapeanalysis.sso_detection_service import SSODetectionService
 from services.driver_manager import DriverManager
 from services.rest_client import RestClient
 
+temp_folder = tempfile.tempdir + "/ssoworker/landscape-analysis/"
+if not os.path.exists(temp_folder):
+    os.makedirs(temp_folder)
+
 
 def thread_process(site, backend_info: BackendInformation, process_type: ProcessType, search_engines,
                    analysis_run_id):
     success = False
     rest_client = None
     chromedriver = None
+    har_files_location = temp_folder + "har-files/"
+    if os.path.exists(har_files_location):
+        logger.info("Found old temp har files directory (may be empty, could come from aborted run). Deleting... ")
+        shutil.rmtree(har_files_location)
+    os.makedirs(har_files_location, exist_ok=True)
     cause = "Unknown"
     try:
         site_no_proto = ProcessHelper.remove_protocol_if_existent(site.base_page)
         rest_client = RestClient(backend_info.host, backend_info.port, backend_info.token)
         known_sso_provider = rest_client.get_known_sso_provider()
+        start_time = time.time()
         logger.info("Received site " + site.base_page + " to analyse. Starting chromedriver...")
         rest_client.update_progress_of_analysis(analysis_run_id, site.base_page, 0, "Starting chromedriver")
         chromedriver = None
@@ -48,6 +63,7 @@ def thread_process(site, backend_info: BackendInformation, process_type: Process
         logger.info("Checking sso support for " + site.base_page + " (id:" + str(site.index) + "|trancoID:" + str(
             rest_client.get_tranco_id_for_site(site.base_page)) + ")")
         login_candidates = []
+        time_login_site_detection = time.time()
         if process_type == ProcessType.AUTOMATIC_SSO_DETECTION_BY_SEARCH_ENGINE:
             if chromedriver is None:
                 chromedriver = DriverManager.generate_driver()
@@ -84,6 +100,7 @@ def thread_process(site, backend_info: BackendInformation, process_type: Process
             rest_client.update_progress_of_analysis(analysis_run_id, site.base_page, 20,
                                                     "Loading login page from brain")
             latest_login_infos = [rest_client.get_latest_login_location_for_page_by_page_id(site.index)]
+        time_login_site_detection = time.time() - time_login_site_detection
         # Finished Login Gathering
         if chromedriver is not None:
             logger.info("Preparations are finished. Closing chrome instance which was started for preparation.")
@@ -91,15 +108,22 @@ def thread_process(site, backend_info: BackendInformation, process_type: Process
         results = []
         if len(latest_login_infos) == 0:
             raise NoLoginCandidatesFoundException()
-        steps = 60 / len(latest_login_infos)
+        steps = 30 / len(latest_login_infos)
         last_progress = 20
         har_files = []
+
+        # KEYWORD ANALYSIS
+        time_combined_analyses = time.time()
+        time_keyword_based_analysis = time.time()
+        logger.info("Starting keyword based analysis")
+        file_counter = 0
         for latest_login_info in latest_login_infos:
             logger.info("Starting analysis for " + latest_login_info.loginPath)
             already_analysed_login_page = False
-            for result in results:
-                if result["info"].loginPath == latest_login_info.loginPath:
-                    results.append({"ids": result["ids"], "info": latest_login_info, "screen": result["screen"]})
+            for res_logo in results:
+                if res_logo["info"].loginPath == latest_login_info.loginPath:
+                    results.append(
+                        {"ids": res_logo["ids"], "info": latest_login_info, "screen": res_logo["screen"]})
                     already_analysed_login_page = True
                     break
             if already_analysed_login_page:
@@ -129,7 +153,8 @@ def thread_process(site, backend_info: BackendInformation, process_type: Process
             search_for_google = not ProcessHelper.contain_results_provider(results, "google", known_sso_provider)
             if not search_for_google:
                 logger.info("Google SSO Support already found. Skipping it in analysis")
-            search_for_facebook = not ProcessHelper.contain_results_provider(results, "facebook", known_sso_provider)
+            search_for_facebook = not ProcessHelper.contain_results_provider(results, "facebook",
+                                                                             known_sso_provider)
             if not search_for_facebook:
                 logger.info("Facebook SSO Support already found. Skipping it in analysis")
             search_for_apple = not ProcessHelper.contain_results_provider(results, "apple", known_sso_provider)
@@ -138,15 +163,88 @@ def thread_process(site, backend_info: BackendInformation, process_type: Process
 
             ids = gather_sso_support(known_sso_provider, latest_login_info, chromedriver, process_type,
                                      search_for_google, search_for_facebook, search_for_apple, progress_callback)
-            logger.info("Analysis finished for " + latest_login_info.loginPath +
+            logger.info("Keyword based analysis finished for " + latest_login_info.loginPath +
                         ". Exporting har and stopping chromedriver")
-            har_files.append(chromedriver.har)
+            file_counter += 1
+            har_file = har_files_location + str(file_counter) + "-keyword-alg-" + ProcessHelper.gen_filename_from_url(
+                latest_login_info.loginPath) + ".har"
+            with open(har_file, mode="w") as file:
+                file.write(chromedriver.har)
+            har_files.append(har_file)
+            logger.info("Exported har to " + har_file)
             ProcessHelper.quit_chromedriver_correctly(chromedriver)
             last_progress = last_progress + steps / 1.5
             results.append({"ids": ids, "info": latest_login_info, "screen": screen})
+        time_keyword_based_analysis = time.time() - time_keyword_based_analysis
+
+        # IMAGE ANALYSIS
+        time_image_analysis = time.time()
+        image_analysis_percent = 30
+        logger.info("Starting logo detection analysis")
+        urls_to_analyse = []
+        for login_candidate in login_candidates:
+            if not urls_to_analyse.__contains__(login_candidate):
+                urls_to_analyse.append(login_candidate['site'])
+
+        def progress_callback_image_analysis(step: int, max: int, status: str):
+            rest_client.update_progress_of_analysis(analysis_run_id, site.base_page,
+                                                    last_progress + step * (image_analysis_percent / max), status)
+
+        logoR = LogoReconizer(temp_folder)
+        res_logo = logoR.analyseHybridLoginURL(domain_url=site.base_page.replace("https://", "").replace("http://", ""),
+                                               login_urls=urls_to_analyse,
+                                               init_recognition={
+                                                   "facebook": ProcessHelper.contain_results_provider(results,
+                                                                                                      "facebook",
+                                                                                                      known_sso_provider),
+                                                   "google": ProcessHelper.contain_results_provider(results, "google",
+                                                                                                    known_sso_provider),
+                                                   "apple": ProcessHelper.contain_results_provider(results, "apple",
+                                                                                                   known_sso_provider)},
+                                               callback=progress_callback_image_analysis)
+
+        for r in res_logo["details"]:
+            logger.info("Logo recognition found following sso provider %s", r["provider"])
+            found = False
+            for orig_results in results:
+                if orig_results['info'].loginPath == r['login_url']:
+                    orig_results['ids'].append((ProcessHelper.get_provider_id_from_remote_known_providers(
+                        known_sso_provider, r['provider']), {'x': math.ceil(r['coordinates'][0]),
+                                                             'y': math.ceil(r['coordinates'][1])},
+                                                None, r['login_url'], SSODetectionStrategy.IMAGE_ANALYSIS,
+                                                r['idp_url']))
+                    logger.info("Exporting har.")
+                    file_counter += 1
+                    har_file = har_files_location + str(file_counter) + "-image-alg-" + r['provider'] + "-" + \
+                               ProcessHelper.gen_filename_from_url(r['login_url']) + ".har"
+                    with open(har_file, mode="w") as file:
+                        file.write(r['har'])
+                    har_files.append(har_file)
+                    logger.info("Exported har to " + har_file)
+                    found = True
+                    break
+            if not found:
+                raise Exception("Could not find a corresponding login candidate for the webpage " + r['login_url'])
+        time_image_analysis = time.time() - time_image_analysis
+        time_combined_analyses = time.time() - time_combined_analyses
+        time_complete_process = time.time() - start_time
         rest_client.update_progress_of_analysis(analysis_run_id, site.base_page, 90, "Uploading results")
         success = save_supported_sso_provider(site, results, har_files, process_type, rest_client, analysis_run_id)
         logger.info("Successfully saved!" if success else "Could not save site")
+        time_complete_process_with_upload = time.time() - start_time
+        logger.info("Timing results:" + ((" LD " + str(
+            time_login_site_detection) + " |") if process_type == ProcessType.AUTOMATIC_SSO_DETECTION_BY_SEARCH_ENGINE else "") + " KBA " + str(
+            time_keyword_based_analysis) + " | LRA " + str(time_image_analysis) + " | CBA " + str(
+            time_combined_analyses / len(latest_login_infos)) + " | CP " + str(
+            time_complete_process) + " | CPWU " + str(time_complete_process_with_upload))
+        logger.info("Timing results per page (" + str(len(latest_login_infos)) + " sites analysed):" + ((" LD " + str(
+            time_login_site_detection / len(
+                latest_login_infos)) + " |") if process_type == ProcessType.AUTOMATIC_SSO_DETECTION_BY_SEARCH_ENGINE else "") + " KBA " + str(
+            time_keyword_based_analysis / len(latest_login_infos)) + " | LRA " + str(
+            time_image_analysis / len(latest_login_infos)) + " | CBA " + str(
+            time_combined_analyses / len(latest_login_infos)) + " | CP " + str(
+            time_complete_process / len(latest_login_infos)) + " | CPWU " + str(
+            time_complete_process_with_upload / len(latest_login_infos)))
     except RenewalRequestNeededException:
         logger.error("Sending renewal request")
         rest_client.create_renew_login_location_request(site.index)
@@ -184,6 +282,9 @@ def thread_process(site, backend_info: BackendInformation, process_type: Process
         if chromedriver is not None:
             ProcessHelper.quit_chromedriver_correctly(chromedriver)
             del chromedriver.requests, chromedriver
+        if os.path.exists(temp_folder):
+            logger.info("Cleaning up temporary folder")
+            shutil.rmtree(temp_folder)
 
 
 # Returns if the current site should be skipped
