@@ -1,5 +1,6 @@
 import base64
 import json
+import os.path
 from time import sleep
 
 import requests
@@ -226,15 +227,19 @@ class RestClient:
         for id in ids:
             tid = [id[0]]
             if len(id) > 1:
-                if id[1] is not None and id[2] is not None:
+                if id[1] is not None:
                     tid.append(id[1]['x'])
                     tid.append(id[1]['y'])
+                else:
+                    tid.extend([None, None])
+                if id[2] is not None:
                     tid.append(id[2]['width'])
                     tid.append(id[2]['height'])
                 else:
-                    tid.extend([None, None, None, None])
+                    tid.extend([None, None])
                 tid.append(id[3])
                 tid.append(id[4].name)
+                tid.append(id[5])
             transformed_ids.append(tid)
         post_data = {'analysisrunid': analysis_session, 'trancosite': trancosite,
                      'supportedproviderids': transformed_ids, "loginpages": []}
@@ -242,14 +247,17 @@ class RestClient:
             for result in results:
                 post_data['loginpages'].append(result['info'].loginPath)
         files = []
+        size = 0
         if har_files is not None and len(har_files) > 0:
             for har_file in har_files:
-                files.append(('har', har_file))
+                size += os.path.getsize(har_file)
+                files.append((os.path.basename(har_file), open(har_file, mode="rb")))
         if results is not None:
             for result in results:
+                size += len(result['screen'])
                 files.append(('screenshot', result['screen']))
         if len(files) > 0:
-            logger.info("Upload size of files (har, screen, ...): " + str(len(str(files)) * 0.000001) + " MB")
+            logger.info("Upload size of files (har, screen, ...): " + str(size * 0.000001) + " MB")
         response = self.send_post_request_with_files(self.path_save_sso_analysis, post_data, files)
         if response.status_code == 200 and response.text == "success":
             logger.info("File uploaded successfully!")
@@ -271,7 +279,7 @@ class RestClient:
         post_data = {'analysisrunid': analysis_session, 'ssodetectionid': sso_det_index}
         files = {}
         if har_content is not None:
-            files['har'] = har_content
+            files['sso_privacy_detection.har'] = har_content
         if screenshot is not None:
             files['screenshot'] = screenshot
         response = self.send_post_request_with_files(self.path_save_privacy_detection, post_data,
@@ -314,7 +322,7 @@ class RestClient:
                      'authres': result[1], 'success': success, 'error': error}
         files = {}
         if har is not None:
-            files['har'] = har
+            files['sso_security_analysis.har'] = har
         if screen is not None:
             files['screenshot'] = screen
         if error_screenshot is not None:
